@@ -16,30 +16,17 @@ use namespace::clean;
 sub _walk_body {
   my ($self, $job_type, $job) = @_;
 
-  my $devices = schema('netdisco')->resultset('Device')->get_column('ip');
   my $jobqueue = schema('netdisco')->resultset('Admin');
+  my $devices = schema('netdisco')->resultset('Device')
+    ->search({ip => { -not_in =>
+        $jobqueue->search({
+          device => { '!=' => undef},
+          action => $job_type,
+          status => { -like => 'queued%' },
+        })->get_column('device')->as_query
+    }})->get_column('ip');
 
-  if ($job->subaction and $job->subaction eq 'after-discoverall') {
-      # make sure there are no incomplete discover jobs queued
-      my $discover = $jobqueue->search(
-        { action => 'discover', status => { -like => 'queued%' } }
-      )->count;
-
-      return job_defer("Deferred $job_type due to pending discover jobs")
-        if $discover;
-  }
-
-  schema('netdisco')->txn_do(sub {
-    # clean up user submitted jobs older than 1min,
-    # assuming skew between schedulers' clocks is not greater than 1min
-    $jobqueue->search({
-        action => $job_type,
-        status => 'queued',
-        entered => { '<' => \"(now() - interval '1 minute')" },
-    })->delete;
-
-    # is scuppered by any user job submitted in last 1min (bad), or
-    # any similar job from another scheduler (good)
+  schema('netdisco')->resultset('Admin')->txn_do_locked(sub {
     $jobqueue->populate([
       map {{
           device => $_,
