@@ -56,7 +56,7 @@ hook 'before' => sub {
 
   # override ports form defaults with cookie settings
 
-  my $cookie = cookie('nd_ports-form');
+  my $cookie = (cookie('nd_ports-form') || '');
   my $cdata = url_params_mixed($cookie);
 
   if ($cdata and ref {} eq ref $cdata and not param('reset')) {
@@ -159,7 +159,8 @@ get '/device' => require_login sub {
     });
 
     if (!defined $dev) {
-        return redirect uri_for('/', {nosuchdevice => 1})->as_string();
+        header(Location => uri_for('/', {nosuchdevice => 1})->path_query);
+        return status(302);
     }
 
     params->{'tab'} ||= 'details';
