@@ -66,7 +66,8 @@ get '/search' => require_login sub {
 
     if (not param('tab')) {
         if (not $q) {
-            return redirect uri_for('/')->path;
+            header(Location => uri_for('/')->path);
+            return status(302);
         }
 
         # pick most likely tab for initial results
@@ -79,11 +80,12 @@ get '/search' => require_login sub {
             if ($nd and $nd->count) {
                 if ($nd->count == 1) {
                     # redirect to device details for the one device
-                    return redirect uri_for('/device', {
+                    header(Location => uri_for('/device', {
                       tab => 'details',
                       q => ($nd->first->dns || $nd->first->ip),
                       f => '',
-                    })->path_query;
+                    })->path_query);
+                    return status(302);
                 }
 
                 # multiple devices
